@@ -15,10 +15,14 @@
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import argparse
+import os
 import sys
+import traceback
 
 
 import subunit
+from subunit import test_results
+import testtools
 
 
 from ucitests import (
@@ -65,6 +69,10 @@ class RunTestsArgParser(argparse.ArgumentParser):
         self.add_argument(
             '--format', '-f', choices=['text', 'subunit'], default='text',
             help='Output format for the test results.')
+        self.add_argument(
+            '--concurrency', '-c', dest='concurrency',
+            default=1, type=int,
+            help='concurrency (number of processes)')
 
     def parse_args(self, args=None, out=None, err=None):
         """Parse arguments, overridding stdout/stderr if provided.
@@ -111,7 +119,7 @@ def cli_run(args=None, stdout=None, stderr=None):
             result = results.TextResult(stdout, verbosity=2)
         else:
             result = subunit.TestProtocolClient(stdout)
-        ret = run_tests(suite, result)
+        ret = run_tests(suite, result, ns.concurrency)
     return ret
 
 
@@ -164,13 +172,15 @@ def list_tests(suite, stream):
     return int(no_tests)
 
 
-def run_tests(suite, result):
+def run_tests(suite, result, process_nb=1):
     """Run the provided tests with the provided test result.
 
     :param suite: A test suite.
 
     :param result: The collecting test result object.
 
+    :param process_nb: The number of processes to split the run across.
+
     :return: 0 on success, 1 otherwise.
 
     :note: Running no tests is an error. The rationale is that when used from a
@@ -181,7 +191,85 @@ def run_tests(suite, result):
     """
     result.startTestRun()
     try:
+        if process_nb > 1:
+            suite = testtools.ConcurrentTestSuite(
+                suite, fork_for_tests(process_nb))
         suite.run(result)
     finally:
         result.stopTestRun()
     return int(not (result.wasSuccessful() and result.testsRun > 0))
+
+
+class TestInOtherProcess(subunit.ProtocolTestCase):
+
+    def __init__(self, stream, pid):
+        super(TestInOtherProcess, self).__init__(stream)
+        self.pid = pid
+
+    def run(self, result):
+        try:
+            super(TestInOtherProcess, self).run(result)
+        finally:
+            pid, status = os.waitpid(self.pid, 0)
+        # GZ 2011-10-18: If status is nonzero, should report to the result
+        #                that something went wrong.
+
+
+def fork_for_tests(process_nb=1):
+    """Fork helper for testtools.ConcurrentTestSuite.
+
+    :param process_nb: number of processes to use.
+
+    :returns: A function expecting a test suite that will be run across several
+        processes.
+    """
+    def do_fork(suite):
+        """Take suite and start up multiple runners by forking.
+
+        :param suite: TestSuite object.
+
+        :return: An iterable of tests processing subunit streams from suites
+            executed in subprocesses.
+        """
+        # testtools.ConcurrentTestSuite wraps tests in a list which prohibits
+        # filter_suite to preserve the original suites used, unwrap it
+        if isinstance(suite, testtools.ConcurrentTestSuite):
+            suite = suite._tests[0]
+        tests = []
+        sub_suites = filters.partition_suite(suite, process_nb)
+        for sub_suite in sub_suites:
+            c2pread, c2pwrite = os.pipe()
+            pid = os.fork()
+            if pid == 0:
+                # Execute the suite, feeding the subunit stream to the caller
+                try:
+                    stream = os.fdopen(c2pwrite, 'wb', 1)
+                    os.close(c2pread)
+                    # Leave stderr and stdout open so we can see test noise
+                    # Close stdin so that the child goes away if it decides to
+                    # read from stdin (otherwise its a roulette to see what
+                    # child actually gets keystrokes for pdb etc).
+                    sys.stdin.close()
+                    result = test_results.AutoTimingTestResultDecorator(
+                        subunit.TestProtocolClient(stream)
+                    )
+                    sub_suite.run(result)
+                except:
+                    # Try and report traceback on stream, but exit with error
+                    # even if stream couldn't be created or something else
+                    # goes wrong.  The traceback is formatted to a string and
+                    # written in one go to avoid interleaving lines from
+                    # multiple failing children.
+                    try:
+                        stream.write(traceback.format_exc())
+                    finally:
+                        os._exit(1)
+                os._exit(0)
+            else:
+                # Process the subunit stream received from the child
+                os.close(c2pwrite)
+                stream = os.fdopen(c2pread, 'rb', 1)
+                test = TestInOtherProcess(stream, pid)
+                tests.append(test)
+        return tests
+    return do_fork
