@@ -20,6 +20,7 @@ import unittest
 
 
 import subunit
+import testtools
 
 
 from ucitests import (
@@ -234,6 +235,41 @@ AssertionError
                           ['fail', 'pass'], result=result)
 
 
+class TestRunTestsConcurrent(unittest.TestCase):
+
+    def run_suite(self, suite):
+        res = results.TextResult(StringIO(), verbosity=0)
+        # Run tests across 2 processes, it's enough to validate that the
+        # plumbing is right between the main process and the subprocesses.
+        concurrent_suite = testtools.ConcurrentTestSuite(
+            suite, runners.fork_for_tests(2))
+        res.startTestRun()
+        concurrent_suite.run(res)
+        res.stopTestRun()
+        return res
+
+    def test_pass(self):
+        res = self.run_suite(fixtures.make_suite(['pass', 'pass']))
+        self.assertTrue(res.wasSuccessful())
+        self.assertEqual(2, res.testsRun)
+        self.assertEqual(0, len(res.errors))
+        self.assertEqual(0, len(res.failures))
+
+    def test_fail(self):
+        res = self.run_suite(fixtures.make_suite(['pass', 'fail']))
+        self.assertFalse(res.wasSuccessful())
+        self.assertEqual(2, res.testsRun)
+        self.assertEqual(0, len(res.errors))
+        self.assertEqual(1, len(res.failures))
+
+    def test_error(self):
+        res = self.run_suite(fixtures.make_suite(['error', 'fail']))
+        self.assertFalse(res.wasSuccessful())
+        self.assertEqual(2, res.testsRun)
+        self.assertEqual(1, len(res.errors))
+        self.assertEqual(1, len(res.failures))
+
+
 class TestOptionParsing(unittest.TestCase):
 
     def setUp(self):
@@ -251,7 +287,7 @@ class TestOptionParsing(unittest.TestCase):
         self.maxDiff = None
         assertions.assertMultiLineAlmostEqual(self, '''\
 usage: uci-run-tests [-h] [--module MODULE] [--exclude EXCLUDE] [--list]
-                     [--format {text,subunit}]
+                     [--format {text,subunit}] [--concurrency CONCURRENCY]
                      [INCLUDE [INCLUDE ...]]
 
 Load and run tests.
@@ -271,6 +307,8 @@ optional arguments:
   --list, -l            List the tests instead of running them.
   --format {text,subunit}, -f {text,subunit}
                         Output format for the test results.
+  --concurrency CONCURRENCY, -c CONCURRENCY
+                        concurrency (number of processes)
 ''',
                                               self.out.getvalue())
 
@@ -281,6 +319,7 @@ optional arguments:
         self.assertEqual(None, ns.exclude_regexps)
         self.assertFalse(ns.list_only)
         self.assertEqual('text', ns.format)
+        self.assertEqual(1, ns.concurrency)
 
     def test_include_regexps(self):
         ns = self.parse_args(['a', 'b'])
