@@ -40,7 +40,7 @@ class TestTestPep8Usage(unittest.TestCase):
             test.test_pep8_conformance()
 
         self.assertEqual('You should define some packages to check',
-                         cm.exception.message)
+                         unicode(cm.exception))
 
 
 @features.requires(tests.minimal_pyflakes)
@@ -57,18 +57,7 @@ class TestTestPyflakesUsage(unittest.TestCase):
             test.test_pyflakes_conformance()
 
         self.assertEqual('You should define some packages to check',
-                         cm.exception.message)
-
-
-class TestIterFiles(unittest.TestCase):
-
-    def setUp(self):
-        super(TestIterFiles, self).setUp()
-        fixtures.set_uniq_cwd(self)
-
-    def test_empty(self):
-        # self.assertEqual([], styles.iter_files([])
-        pass
+                         unicode(cm.exception))
 
 
 class TestPythonWalker(unittest.TestCase):
